
ALTER DATABASE pubs SET HONOR_BROKER_PRIORITY ON;
GO

USE pubs
go

CREATE CONTRACT premier
    ([DEFAULT] SENT BY ANY)
go

CREATE QUEUE receiveq
go

-- default means receiver can receive 
-- message type DEFAULT
-- Premier is another contract for Premier Service
CREATE SERVICE receiver
	ON QUEUE receiveq
	([DEFAULT], premier)
go

-- You can also add the premier contract in after the fact
-- ALTER SERVICE receiver ON QUEUE receiveq (ADD CONTRACT premier);

CREATE QUEUE senderq 
go

-- no contract because sender names the contract
-- in begin dialog
CREATE SERVICE sender
	ON QUEUE senderq
go

CREATE BROKER PRIORITY PremierCustomer
    FOR CONVERSATION
    SET (CONTRACT_NAME = premier,
         LOCAL_SERVICE_NAME = sender,
         REMOTE_SERVICE_NAME = 'receiver',
         PRIORITY_LEVEL = 7);
GO

-- note that you need both priorities
-- if you don't have this one, the sender's endpoint and messages TO the sender are priority 7
-- but the receiver's endpoint and messages TO the receiver are priority 5
CREATE BROKER PRIORITY PremierCustomer2
    FOR CONVERSATION
    SET (CONTRACT_NAME = premier,
         LOCAL_SERVICE_NAME = receiver,
         REMOTE_SERVICE_NAME = 'sender',
         PRIORITY_LEVEL = 6);
GO

DECLARE @h UNIQUEIDENTIFIER;
BEGIN TRAN;
BEGIN TRY
	BEGIN DIALOG CONVERSATION @h
	FROM SERVICE sender
	TO SERVICE 'receiver'
	ON CONTRACT [DEFAULT]   -- contract [default] is the default contract
	WITH ENCRYPTION = OFF;

	SEND ON CONVERSATION @h
	('Hello World');
    COMMIT;
END TRY
BEGIN CATCH
	PRINT ERROR_MESSAGE()
    ROLLBACK;
END CATCH
-- don't end conversation here
-- END CONVERSATION @h;
go

-- note: both endpoints have priority 5
SELECT * FROM sys.conversation_endpoints;
Go

-- note the message is in the receiveq waiting to be read
-- the default message priority is 5
SELECT * FROM receiveq; 
GO

DECLARE @h UNIQUEIDENTIFIER;
BEGIN TRAN;
BEGIN TRY
	BEGIN DIALOG CONVERSATION @h
	FROM SERVICE sender
	TO SERVICE 'receiver'
	ON CONTRACT [premier]   
	WITH ENCRYPTION = OFF;

	SEND ON CONVERSATION @h
	('Hello World from a Premier Customer');
    COMMIT;
END TRY
BEGIN CATCH
	PRINT ERROR_MESSAGE()
    ROLLBACK;
END CATCH
-- don't end conversation here
-- END CONVERSATION @h;
go

-- note: the sender's conversation endpoint has priority 7
--       the receiver's conversation endpoint has priority 6
SELECT * FROM sys.conversation_endpoints;
GO

-- note: the premier message is at priority 6 (not 7)
SELECT * FROM receiveq;
GO

DECLARE @h UNIQUEIDENTIFIER;
DECLARE @m VARCHAR(MAX);
BEGIN TRAN;
RECEIVE TOP(1) @h=conversation_handle, 
		@m=CAST(message_body as VARCHAR(MAX)) FROM receiveq;
PRINT @m;
-- dont ever end conversation with cleanup in a program
-- END CONVERSATION @h WITH CLEANUP;
END CONVERSATION @h
COMMIT;
go

-- note: the return message is at priority 7
SELECT * FROM senderq;
GO

-- receive end of conversation at the sender
-- and end the conversation there
DECLARE @h UNIQUEIDENTIFIER;
DECLARE @m VARCHAR(MAX);
BEGIN TRAN;
RECEIVE TOP(1) @h=conversation_handle, 
		@m=CAST(message_body as VARCHAR(MAX)) FROM senderq;
PRINT @m;
END CONVERSATION @h
COMMIT;
go

-- messages sent by the sender are priority 7 
-- while they are sending
DECLARE @h UNIQUEIDENTIFIER;
BEGIN TRAN;
BEGIN TRY
	BEGIN DIALOG CONVERSATION @h
	FROM SERVICE sender
	TO SERVICE 'receiver'
	ON CONTRACT [premier]   
	WITH ENCRYPTION = ON;   -- this sticks in the transmission queue,
	                        -- because we're missing a database master key

	SEND ON CONVERSATION @h
	('Hello World from a Premier Customer');
    COMMIT;
END TRY
BEGIN CATCH
	PRINT ERROR_MESSAGE()
    ROLLBACK;
END CATCH
-- don't end conversation here
-- END CONVERSATION @h;
go

select * from sys.transmission_queue;


-- better, have an activation program on senderq
-- that processes end conversation and errors

CREATE PROCEDURE inititator_queue_activated_procedure
AS
DECLARE @dh UNIQUEIDENTIFIER;
DECLARE @message_type SYSNAME;
DECLARE @message_body NVARCHAR(4000);
BEGIN TRANSACTION;
WAITFOR (
      RECEIVE @dh = [conversation_handle],
            @message_type = [message_type_name],
            @message_body = CAST([message_body] AS NVARCHAR(4000)) 
      FROM senderq), TIMEOUT 1000;
WHILE @dh IS NOT NULL
BEGIN
      IF @message_type = N'http://schemas.microsoft.com/SQL/ServiceBroker/Error'
      BEGIN
            RAISERROR (N'Received error %s from service [receiver]', 10, 1, @message_body) WITH LOG;
      END
      END CONVERSATION @dh;
      COMMIT;
      SELECT @dh = NULL;
      BEGIN TRANSACTION;
      WAITFOR (
            RECEIVE @dh = [conversation_handle],
                  @message_type = [message_type_name],
                  @message_body = CAST([message_body] AS NVARCHAR(4000)) 
            FROM senderq), TIMEOUT 1000;
END
COMMIT;
GO
 
ALTER QUEUE senderq
      WITH ACTIVATION (
            STATUS = ON,
            MAX_QUEUE_READERS = 1,
            PROCEDURE_NAME = [inititator_queue_activated_procedure],
            EXECUTE AS OWNER);
GO



